package com.yem.wm.im.inquirylist.form;

import com.yem.wm.utils.FunctionalCommon;
import com.yem.wm.utils.OrgFilterUtils;
import com.yem.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.mvc.bill.BillView;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author power
 * @ClassName InquirylistEdit
 * @description: 询价单表单插件
 * @date 2023年07月25日
 * @version: 1.0
 */
public class InquirylistEdit extends AbstractBillPlugIn implements BeforeF7SelectListener {
    public static final String YEM_BD_PRODUCTS = "yem_bd_products";

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs e) {
        super.afterDoOperation(e);
        IFormView view = this.getView();
        IDataModel model = this.getModel();
        String operateKey = e.getOperateKey();
        OperationResult operationResult = e.getOperationResult();
        switch (operateKey) {
            case "optionaloperation":
                optionalOperation(model, view, "yem_optional", "optionaloperation");
                break;
            case "replaceoperation":
                optionalOperation(model, view, "yem_replace", "replaceoperation");
                break;
            case "retrofittingoperation":
                optionalOperation(model, view, "yem_retrofitting", "retrofittingoperation");
                break;
//           case "submit":
////                if (YEM.isNotEmpty(operationResult)) {
////                    view.invokeOperation("refresh");
////                }
//                break;
//            case "unsubmit":
//                if (YEM.isNotEmpty(operationResult)) {
//                    view.invokeOperation("refresh");
//                }
//                break;
        }
    }


    /**
     * @description 选装、替换、加装功能公共方法
     * @author xwudd
     * @param[1] model
     * @param[2] view
     * @param[3] formId 动态表单标识
     * @param[4] operationNumber 服务标识
     * return
     * @time 2023/7/27 9:38
     */

    public static void optionalOperation(IDataModel model, IFormView view, String formId, String operationNumber) {
        String name = model.getDataEntity().getDataEntityType().getName();
        int currentRowIndex = -1;
        if ("yem_es_salesorder".equals(name) || "yem_es_xsalesorder".equals(name)) {
            currentRowIndex = model.getEntryCurrentRowIndex("yem_es_materialinfo");
        }
        if ("yem_im_clientdemand".equals(name) || "yem_im_inquirylist".equals(name)) {
            currentRowIndex = model.getEntryCurrentRowIndex("yem_im_detailedinfor");
        }
        DynamicObject billType = (DynamicObject) model.getValue("yem_billtype");

        DynamicObjectCollection yem_im_detailedinfor = model.getEntryEntity("yem_im_detailedinfor");
        Map<String, Object> map = new HashMap<>();
        List<Long> ids = new ArrayList();
        DynamicObject productModel = (DynamicObject) model.getValue("yem_productmodel", currentRowIndex);
        DynamicObject products = yem_im_detailedinfor.get(0).getDynamicObject("yem_productmodel.yem_products");
        for (DynamicObject dynamicObject : yem_im_detailedinfor) {
            DynamicObjectCollection yem_multiconfig = dynamicObject.getDynamicObjectCollection("yem_multiconfig");
            if (yem_multiconfig != null) {

//                int multiConfigRows = model.getEntryRowCount("yem_multiconfig");
                for (DynamicObject object : yem_multiconfig) {
//                    String configurType = (String) model.getValue("yem_configurtype", i);
                    Long configEntryId = object.getLong("yem_configentryid");
                    ids.add(configEntryId);
                }


            }
        }
        map.put("billTypeNumber", billType.getString("number"));
        map.put("products", products.getPkValue());
        map.put("operationNumber", operationNumber);
        map.put("type", "B");
        map.put("ids", ids);
        map.put("productModel", productModel);
        viewForm(formId, map, view);
    }


    /**
     * @description 打开动态表单
     * @author xwudd
     * @param[1] formId 表单标识
     * @param[2] map 传参     无参传null
     * @param[3] view
     * return
     * @time 2023/8/29 11:19
     */

    public static void viewForm(String formId, Map<String, Object> map, IFormView view) {
        FormShowParameter formShowParameter = new FormShowParameter();
        formShowParameter.setFormId(formId);
        formShowParameter.getOpenStyle().setShowType(ShowType.Modal);
        if (YEM.isNotEmpty(map)) {
            formShowParameter.setCustomParams(map);
        }
        view.showForm(formShowParameter);
    }


    /**
     * @param isDisable
     * @Description 禁用操作
     */
    private void disableOperation(boolean isDisable) {
        IDataModel model = this.getModel();
        EntryGrid multiconfig = this.getControl("yem_multiconfig");

        int detailedinforSelectRow = model.getEntryCurrentRowIndex("yem_im_detailedinfor");

        int[] multiconfigSelectRows = multiconfig.getSelectRows();

        DynamicObject dataEntity = model.getDataEntity();

        DynamicObjectCollection collection = dataEntity.getDynamicObjectCollection("yem_im_detailedinfor");
        DynamicObjectCollection subCollection = collection.get(detailedinforSelectRow).getDynamicObjectCollection("yem_multiconfig");
        for (int subRow : multiconfigSelectRows) {
            DynamicObject dynamicObject = subCollection.get(subRow);
            if (isDisable) {
                // 禁用
                dynamicObject.set("yem_disabledstate", "B");
                dynamicObject.set("yem_prohibitedperson", BusinessDataServiceHelper.loadSingle(YEM.getCurrentUserId(), "bos_user"));
                dynamicObject.set("yem_disabletime", new Date());
            } else {
                // 启用
                dynamicObject.set("yem_disabledstate", "A");
                dynamicObject.set("yem_prohibitedperson", null);
                dynamicObject.set("yem_disabletime", null);
            }
            subCollection.set(subRow, dynamicObject);
        }
        this.getView().updateView("yem_multiconfig");
    }

    @Override
    public void afterCreateNewData(EventObject e) {

        setBillType();
    }


    @Override
    public void registerListener(EventObject e) {
        BasedataEdit products = this.getView().getControl("yem_bd_products");
        if (products != null) {
            products.addBeforeF7SelectListener(this);
        }
        BasedataEdit productModel = this.getControl("yem_productmodel");
        productModel.addBeforeF7SelectListener(this);

        BasedataEdit yem_companyorg = this.getControl("yem_companyorg");
        yem_companyorg.addBeforeF7SelectListener(this);
    }

    /**
     * @Description 过滤产品类型配件 整机
     */

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        DynamicObject datas = this.getModel().getDataEntity(true);
        String key = e.getProperty().getName();
        QFilter qFilter = null;
        DynamicObject products = datas.getDynamicObject(YEM_BD_PRODUCTS);
        switch (key) {
            //过滤产品分类
            case "yem_bd_products":
                DynamicObject types = datas.getDynamicObject("yem_billtype");
                if (types == null) {
                    this.getView().showMessage("请选择单据类型");
                    e.setCancel(true);
                } else {
                    qFilter = new QFilter("yem_businesssort", QCP.equals,
                            types.getString("number").equals("yem_im_inquirylist_ZJ") ? "Z" : "P");
                    e.addCustomQFilter(qFilter);
                }
                break;
            case "yem_productmodel":
                selectF7ProductModel(e);//
                break;
            case "yem_companyorg"://财务组织
                DynamicObject value = (DynamicObject) this.getModel().getValue("org");
                e.addCustomQFilter(OrgFilterUtils.getOrgFilter(value));
                break;
            default:
                break;
        }

    }

    /**
     * @description 选择基础机型根据表头产品分类过滤
     * @author xwudd
     * * @param[1] e
     * @time 2023/7/28 18:09
     */

    public void selectF7ProductModel(BeforeF7SelectEvent e) {
        IFormView view = this.getView();
        IDataModel model = this.getModel();
        DynamicObject products = (DynamicObject) model.getValue("yem_bd_products");
        if (products != null) {
            e.addCustomQFilter(new QFilter("yem_productsgroup", QCP.equals, products.getPkValue()));
        } else {
            view.showErrorNotification("请先维护基本信息-产品分类字段!");
            e.setCancel(true);

        }
    }

    /**
     * @Description 根据布局给
     */
    public void setBillType() {

        IFormView view = this.getView();
        IDataModel model = this.getModel();
        String key = ((BillView) this.getView()).getRootControl().getKey();
        QFilter qFilter = null;
        if (key.equals("yem_im_inquirylist_pj")) {
            qFilter = new QFilter("number", QCP.equals, "yem_im_inquirylist_PJ");
        } else if (key.equals("yem_im_inquirylist_zj")) {
            qFilter = new QFilter("number", QCP.equals, "yem_im_inquirylist_ZJ");
        }
        if (qFilter != null) {
            DynamicObject billType = BusinessDataServiceHelper.loadSingle("bos_billtype", "id", qFilter.toArray());
            model.setValue("yem_billtype", billType);
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        IDataModel model = this.getModel();
        String name = e.getProperty().getName();
        int rowIndex = e.getChangeSet()[0].getRowIndex();
        switch (name) {
            case "yem_productmodel":
                model.deleteEntryData("yem_multiconfig");
//                price(rowIndex);
                break;
            case "yem_material":

                DynamicObjectCollection entryEntity = model.getEntryEntity("yem_im_detailedinfor");
                DynamicObject dynamicObject = entryEntity.get(rowIndex);
                long materialId = dynamicObject.getLong("yem_material.id");
                if (materialId > 0) {
                    String number = dynamicObject.getString("yem_material.number");
                    if (number != null && !number.isEmpty()) {
                        DynamicObject materialsort = BusinessDataServiceHelper.loadSingle("yem_bd_materialsort",
                                new QFilter[]{new QFilter("yem_material.number", QCP.equals, number)});
                        if (materialsort != null) {
                            model.setValue("yem_classification", materialsort.getLong("yem_markup.id"), rowIndex);
                        }
                    }
                }
                Object newValue = e.getChangeSet()[0].getNewValue();
                if (newValue == null) {
                    model.setValue("yem_classification", null, rowIndex);
                }
                break;
//            case"yem_qingdaounitprice":
//                price(rowIndex);
        }
    }

    private void price(int rowIndex) {
        DynamicObject yemImDetailedinfor = this.getModel().getEntryEntity("yem_im_detailedinfor").get(rowIndex);
        DynamicObject yemProductmodel = yemImDetailedinfor.getDynamicObject("yem_productmodel");
        BigDecimal yemQingdaounitprice = yemImDetailedinfor.getBigDecimal("yem_qingdaounitprice");
        if (yemQingdaounitprice.compareTo(BigDecimal.ZERO) == 0) {
            return;
        }
        if (yemProductmodel != null) {
            DynamicObject yemProducts = yemProductmodel.getDynamicObject("yem_products");
            if (yemProducts != null) {
                DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle("yem_bd_productstree", "id,number,yem_costdifference"
                        , new QFilter[]{new QFilter("id", QCP.equals, yemProducts.getLong("id"))});
                BigDecimal yemCostdifference = loadSingle.getBigDecimal("yem_costdifference");
                this.getModel().setValue("yem_unitprice", yemQingdaounitprice.subtract(yemCostdifference), rowIndex);
            }
        }
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        DynamicObjectCollection yemImDetailedinfor = (DynamicObjectCollection) this.getModel().getValue("yem_im_detailedinfor");
        if (yemImDetailedinfor.size() > 0) {
            List<Integer> list = new ArrayList<>();
            for (int i = 0; i < yemImDetailedinfor.size(); i++) {
                DynamicObject dynamicObject = yemImDetailedinfor.get(i);
                BigDecimal yemFobprice = dynamicObject.getBigDecimal("yem_fobprice");
                if (yemFobprice.compareTo(BigDecimal.ZERO) != 0) {
                    list.add(i);
                }
            }
            for (int i = list.size(); i != 0; i--) {
                Integer integer = list.get(i - 1);
                this.getModel().deleteEntryRow("yem_im_detailedinfor", integer);
            }
        }
    }
}
